@@ -18,28 +18,29 @@
  *
  * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
  *
- $Id: battstat_applet.c,v 1.53.2.2 2003/10/26 01:11:29 kmaraas Exp $
+ $Id: battstat_applet.c,v 1.66 2004/02/29 08:19:17 dsmit Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <sys/file.h>
-#include <sys/ioctl.h>
-#include <sys/types.h>
-#include <sys/sysctl.h>
 #include <stdio.h>
 
 #ifdef __FreeBSD__
 #include <machine/apm_bios.h>
-#elif __OpenBSD__
+#elif defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/param.h>
 #include <machine/apmvar.h>
 #elif __linux__
 #include <apm.h>
 #endif
 
+#include <sys/file.h>
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/sysctl.h>
+
 #include <err.h>
 #include <stdlib.h>
 #include <string.h>
@@ -66,6 +67,10 @@
 #define gettext_noop(String) (String)
 #endif
 
+#if defined(__NetBSD__)
+#define APMDEVICE "/dev/apm"
+#endif
+
 GtkObject *statusdock;
 
 static const BonoboUIVerb battstat_menu_verbs [] = {
@@ -215,7 +220,7 @@ GdkColor darkred[] = {
   {-1,0x0000,0x0000,0x0000}
 };
 
-#ifdef __OpenBSD__
+#if defined(__NetBSD__) || defined(__OpenBSD__)
 struct apm_power_info apminfo;
 #else
 struct apm_info apminfo;
@@ -246,7 +251,7 @@ apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 
   close(fd);
 }
-#elif __OpenBSD__
+#elif defined(__NetBSD__) || defined(__OpenBSD__)
 void
 apm_readinfo (PanelApplet *applet, ProgressData * battstat)
 {
@@ -254,7 +259,11 @@ apm_readinfo (PanelApplet *applet, ProgressData * battstat)
      procedure as for FreeBSD.
   */
   int fd;
+#if defined(__NetBSD__)
+  if (DEBUG) g_print("apm_readinfo() (NetBSD)\n");
+#else /* __OpenBSD__ */
   if (DEBUG) g_print("apm_readinfo() (OpenBSD)\n");
+#endif
 
   fd = open(APMDEVICE, O_RDONLY);
   if (fd == -1) cleanup (applet, 1);
@@ -392,7 +401,7 @@ pixmap_timeout( gpointer data )
    batt_state = apminfo.ai_batt_stat;
    batt_life = apminfo.ai_batt_life;
    charging = (batt_state == 3) ? TRUE : FALSE;
-#elif __OpenBSD__
+#elif defined(__NetBSD__) || defined(__OpenBSD__)
    acline_status = apminfo.ac_state ? 1 : 0;
    batt_state = apminfo.battery_state;
    batt_life = apminfo.battery_life;
@@ -429,11 +438,11 @@ pixmap_timeout( gpointer data )
    ;
 
    if ( pixmap_index != battery->last_pixmap_index ) {
-      gtk_pixmap_set(GTK_PIXMAP (battery->statuspixmapwid),
-                     statusimage[pixmap_index], statusmask[pixmap_index]);
+      gtk_image_set_from_pixmap(GTK_IMAGE (battery->statuspixmapwid),
+				statusimage[pixmap_index], statusmask[pixmap_index]);
 #ifdef FIXME
-      gtk_pixmap_set(GTK_PIXMAP (battery->pixmapdockwid),
-                     statusimage[pixmap_index], statusmask[pixmap_index]);
+      gtk_image_set_from_pixmap(GTK_IMAGE (battery->pixmapdockwid),
+				statusimage[pixmap_index], statusmask[pixmap_index]);
 #endif
    }
 
@@ -537,19 +546,19 @@ pixmap_timeout( gpointer data )
       
       if(battery->showbattery) {
 	 if(battery->horizont)
-	   gdk_draw_pixmap(battery->pixmap,
-			   battery->pixgc,
-			   battery->pixbuffer,
-			   0,0,
-			   0,0,
-			   -1,-1);
+	   gdk_draw_drawable(battery->pixmap,
+			     battery->pixgc,
+			     battery->pixbuffer,
+			     0,0,
+			     0,0,
+			     -1,-1);
 	 else
-	   gdk_draw_pixmap(battery->pixmapy,
-			   battery->pixgc,
-			   battery->pixbuffery,
-			   0,0,
-			   0,0,
-			   -1,-1);
+	   gdk_draw_drawable(battery->pixmapy,
+			     battery->pixgc,
+			     battery->pixbuffery,
+			     0,0,
+			     0,0,
+			     -1,-1);
 	 
 	 if(battery->draintop) {
 	    progress_value = PROGLEN*batt_life/100.0;
@@ -611,24 +620,24 @@ pixmap_timeout( gpointer data )
 
 	 if(battery->horizont) {
 	    GdkPixmap *pixmap;
-	    gtk_pixmap_get (GTK_PIXMAP (battery->pixmapwid), &pixmap, NULL);
-	    gdk_draw_pixmap(pixmap,
-			    battery->pixgc,
-			    battery->pixmap,
-			    0,0,
-			    0,0,
-			    -1,-1);
+	    gtk_image_get_pixmap (GTK_IMAGE (battery->pixmapwid), &pixmap, NULL);
+	    gdk_draw_drawable(pixmap,
+			      battery->pixgc,
+			      battery->pixmap,
+			      0,0,
+			      0,0,
+			      -1,-1);
 	    gtk_widget_queue_draw (GTK_WIDGET (battery->pixmapwid));
 	 }
 	 else {
 	    GdkPixmap *pixmap;
-	    gtk_pixmap_get (GTK_PIXMAP (battery->pixmapwidy), &pixmap, NULL);
-	    gdk_draw_pixmap(pixmap,
-			    battery->pixgc,
-			    battery->pixmapy,
-			    0,0,
-			    0,0,
-			    -1,-1);
+	    gtk_image_get_pixmap (GTK_IMAGE (battery->pixmapwidy), &pixmap, NULL);
+	    gdk_draw_drawable(pixmap,
+			      battery->pixgc,
+			      battery->pixmapy,
+			      0,0,
+			      0,0,
+			      -1,-1);
 	    gtk_widget_queue_draw (GTK_WIDGET (battery->pixmapwidy));
 	 }
       }
@@ -712,6 +721,9 @@ destroy_applet (GtkWidget *widget, gpointer data)
    
    if (DEBUG) g_print("destroy_applet()\n");
 
+   if (pdata->about_dialog)
+	gtk_widget_destroy (pdata->about_dialog);
+
    if (pdata->prop_win)
 	gtk_widget_destroy (GTK_WIDGET (pdata->prop_win));
 
@@ -722,17 +734,16 @@ destroy_applet (GtkWidget *widget, gpointer data)
    g_object_unref(pdata->ac_tip);
    g_object_unref(pdata->progress_tip);
    g_object_unref(pdata->progressy_tip);
-   g_object_unref(pdata->testpixgc);
-   gdk_pixmap_unref(pdata->pixmap);
-   gdk_pixmap_unref(pdata->pixmapy);
-   gdk_pixmap_unref(pdata->pixbuffer);
-   gdk_bitmap_unref(pdata->mask);
-   gdk_bitmap_unref(pdata->masky);
-   gdk_bitmap_unref(pdata->pixmask);
-
+   g_object_unref(pdata->pixmap);
+   g_object_unref(pdata->pixmapy);
+   g_object_unref(pdata->pixbuffer);
+   g_object_unref(pdata->mask);
+   g_object_unref(pdata->masky);
+   g_object_unref(pdata->pixmask);
 #ifdef __linux__
    if (using_acpi) {
-     g_source_remove(pdata->acpiwatch);
+     if (pdata->acpiwatch != 0)
+         g_source_remove(pdata->acpiwatch);
      pdata->acpiwatch = 0;
      acpi_linux_cleanup(&acpiinfo);
    }
@@ -843,7 +854,10 @@ suspend_cb (BonoboUIComponent *uic,
 		      msg = g_strdup_printf(_("An error occured while launching the Suspend command: %s\nPlease try to correct this error"), err->message);
 	      } else {
 		      /* Probably because the shell_ret is != 0 */
-		      msg = g_strdup_printf(_("An error occured while launching the Suspend command, the command returned \"%d\"\nPlease try to correct this error"), shell_ret);
+		      if (battstat->suspend_cmd)
+		          msg = g_strdup_printf(_("The Suspend command '%s' was unsuccessful."), battstat->suspend_cmd);
+		      else
+		          msg = g_strdup_printf(_("The Suspend command was unsuccessful."));
 	      }
 	      battstat_error_dialog (PANEL_APPLET (battstat->applet), msg);
 	      g_free(msg);
@@ -857,24 +871,11 @@ suspend_cb (BonoboUIComponent *uic,
    return;
 }
 
-void
-destroy_about (GtkWidget *w, gpointer data)
-{
-   ProgressData *battstat = data;
-   
-   if (DEBUG) g_print("destroy_about()\n");
-   
-   battstat->about_box = NULL;
-   
-   return;
-}
-
 void
 about_cb (BonoboUIComponent *uic,
 	  ProgressData      *battstat,
 	  const char        *verb)
 {
-   GtkWidget   *about_box;
    GdkPixbuf   *pixbuf;
    GError      *error = NULL;
    gchar       *file;
@@ -889,10 +890,20 @@ about_cb (BonoboUIComponent *uic,
    };
 
    const gchar *documenters[] = {
+        "Jorgen Pehrson <jp@spektr.eu.org>",
+        "Trevor Curtis <tcurtis@somaradio.ca>",
 	NULL
    };
 
    const gchar *translator_credits = _("translator_credits");
+
+   if (battstat->about_dialog) {
+	gtk_window_set_screen (GTK_WINDOW (battstat->about_dialog),
+			       gtk_widget_get_screen (GTK_WIDGET (battstat->applet)));
+	   
+	gtk_window_present (GTK_WINDOW (battstat->about_dialog));
+	return;
+   }
    
    file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "battstat.png", FALSE, NULL);
    pixbuf = gdk_pixbuf_new_from_file (file, &error);
@@ -903,7 +914,7 @@ about_cb (BonoboUIComponent *uic,
 	g_error_free (error);
    }
    
-   about_box = gnome_about_new (
+   battstat->about_dialog = gnome_about_new (
 				/* The long name of the applet in the About dialog.*/
 				_("Battery Charge Monitor"), 
 				VERSION,
@@ -917,10 +928,16 @@ about_cb (BonoboUIComponent *uic,
    if (pixbuf)
 	g_object_unref (pixbuf);
 
-   gtk_window_set_wmclass (GTK_WINDOW (about_box), "battery charge monitor", "Batter Charge Monitor");
-   gtk_window_set_screen (GTK_WINDOW (about_box),
+   gtk_window_set_wmclass (GTK_WINDOW (battstat->about_dialog), "battery charge monitor", "Batter Charge Monitor");
+   gtk_window_set_screen (GTK_WINDOW (battstat->about_dialog),
 			  gtk_widget_get_screen (battstat->applet));
-   gtk_widget_show (about_box);
+
+   g_signal_connect (battstat->about_dialog,
+		     "destroy",
+		     G_CALLBACK (gtk_widget_destroyed),
+		     &battstat->about_dialog);
+
+   gtk_widget_show (battstat->about_dialog);
 }
 
 void
@@ -953,7 +970,7 @@ change_orient (PanelApplet       *applet,
    acline_status = apminfo.ai_acline ? 1 : 0;
    batt_state = apminfo.ai_batt_stat;
    batt_life = apminfo.ai_batt_life;
-#elif __OpenBSD__
+#elif defined(__NetBSD__) || defined(__OpenBSD__)
    acline_status = apminfo.ac_state ? 1 : 0;
    batt_state = apminfo.battery_state;
    batt_life = apminfo.battery_life;
@@ -1139,19 +1156,6 @@ change_orient (PanelApplet       *applet,
    return;
 }
 
-
-void
-font_set_cb (GtkWidget *ignored, int page, gpointer data)
-{
-   ProgressData *battstat = data;
-   
-   if (DEBUG) g_print("font_set_cb()\n");
-   battstat->font_changed = TRUE;
-   gnome_property_box_changed (GNOME_PROPERTY_BOX (battstat->prop_win));
-   
-   return;
-}
-
 void
 simul_cb (GtkWidget *ignored, gpointer data)
 {
@@ -1186,12 +1190,12 @@ simul_cb (GtkWidget *ignored, gpointer data)
       darkcolor=darkgreen;
    }
    
-   gdk_draw_pixmap(battery->testpixmap,
-		   battery->testpixgc,
-		   battery->testpixbuffer,
-		   0,0,
-		   0,0,
-		   -1,-1);
+   gdk_draw_drawable(battery->testpixmap,
+		     battery->testpixgc,
+		     battery->testpixbuffer,
+		     0,0,
+		     0,0,
+		     -1,-1);
    if(battery->draintop) {
       progress_value = PROGLEN*slidervalue/100.0;
       
@@ -1274,6 +1278,37 @@ change_size(PanelApplet *applet, gint size, gpointer data)
    battstat->colors_changed=FALSE;
 }
 
+void
+change_background(PanelApplet *a, PanelAppletBackgroundType type, GdkColor *color, GdkPixmap *pixmap, ProgressData *battstat)
+{
+	GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+	switch (type) {
+		case PANEL_PIXMAP_BACKGROUND:
+			/* FIXME use pixmap here */
+			gtk_widget_modify_style (GTK_WIDGET (battstat->applet), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (battstat->hbox), rc_style);
+			break;
+
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (battstat->applet), GTK_STATE_NORMAL, color);
+			gtk_widget_modify_bg (GTK_WIDGET (battstat->hbox), GTK_STATE_NORMAL, color);
+			break;
+
+		case PANEL_NO_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (battstat->applet), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (battstat->hbox), rc_style);
+			break;
+
+		default:
+			gtk_widget_modify_style (GTK_WIDGET (battstat->applet), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (battstat->hbox), rc_style);
+			break;
+	}
+
+	gtk_rc_style_unref (rc_style);
+}
+
 static gboolean
 button_press_cb (GtkWidget *widget, GdkEventButton *event, ProgressData *battstat)
 {
@@ -1395,10 +1430,10 @@ create_layout(ProgressData *battstat)
 							battery_y_gray_xpm ); 
    battstat->pixgc=gdk_gc_new(battstat->pixmap);
    
-   battstat->pixmapwid = gtk_pixmap_new( battstat->pixmap,
-					 battstat->mask );
-   battstat->pixmapwidy = gtk_pixmap_new( battstat->pixmapy,
-					  battstat->masky );
+   battstat->pixmapwid = gtk_image_new_from_pixmap( battstat->pixmap,
+						    battstat->mask );
+   battstat->pixmapwidy = gtk_image_new_from_pixmap( battstat->pixmapy,
+						     battstat->masky );
    gtk_box_pack_start (GTK_BOX (battstat->hbox), GTK_WIDGET (battstat->pixmapwid), FALSE, TRUE, 0);
    gtk_widget_show ( GTK_WIDGET (battstat->pixmapwid) );
 
@@ -1414,8 +1449,10 @@ create_layout(ProgressData *battstat)
 
    gtk_widget_show ( GTK_WIDGET (battstat->pixmapwidy) );
    
-   gtk_signal_connect(GTK_OBJECT(battstat->pixmapwid), "destroy",
-		      GTK_SIGNAL_FUNC(destroy_applet), battstat);
+   g_signal_connect(G_OBJECT(battstat->pixmapwid),
+		    "destroy",
+		    G_CALLBACK(destroy_applet),
+		    battstat);
    
    battstat->percent = gtk_label_new ("0%");
    gtk_widget_show(battstat->percent);
@@ -1440,7 +1477,7 @@ create_layout(ProgressData *battstat)
 							&battstat->style->bg[GTK_STATE_NORMAL],
 							warning_small_xpm );
    battstat->status=statusimage[BATTERY];
-   battstat->statuspixmapwid = gtk_pixmap_new( battstat->status, statusmask[BATTERY] );
+   battstat->statuspixmapwid = gtk_image_new_from_pixmap ( battstat->status, statusmask[BATTERY] );
    gtk_box_pack_start (GTK_BOX (battstat->statusvbox), battstat->statuspixmapwid, FALSE, TRUE, 0);
    gtk_widget_show ( battstat->statuspixmapwid );
 
@@ -1501,16 +1538,30 @@ create_layout(ProgressData *battstat)
 			 "",
 			 NULL);
 
-   gtk_signal_connect(GTK_OBJECT(battstat->applet),"change_orient",
-		      GTK_SIGNAL_FUNC(change_orient),
-		      battstat);
-   g_signal_connect (G_OBJECT (battstat->applet), "change_size",
-   		     G_CALLBACK (change_size), battstat);
-   		     
-   g_signal_connect (battstat->applet, "button_press_event",
-   			     G_CALLBACK (button_press_cb), battstat);
-   g_signal_connect (battstat->applet, "key_press_event",
-   			     G_CALLBACK (key_press_cb), battstat);
+   g_signal_connect (battstat->applet,
+		     "change_orient",
+		     G_CALLBACK(change_orient),
+		     battstat);
+
+   g_signal_connect (battstat->applet,
+		     "change_size",
+   		     G_CALLBACK (change_size),
+		     battstat);
+
+   g_signal_connect (battstat->applet,
+		     "change_background",
+		     G_CALLBACK (change_background),
+		     battstat);
+
+   g_signal_connect (battstat->applet,
+		     "button_press_event",
+   		     G_CALLBACK (button_press_cb),
+		     battstat);
+
+   g_signal_connect (battstat->applet,
+		     "key_press_event",
+		     G_CALLBACK (key_press_cb),
+		     battstat);
 
    return FALSE;
 }
@@ -1520,6 +1571,7 @@ battstat_applet_fill (PanelApplet *applet)
 {
   ProgressData *battstat;
   struct stat statbuf;
+  AtkObject *atk_widget;
 
   if (DEBUG) g_print("main()\n");
   
@@ -1542,9 +1594,9 @@ battstat_applet_fill (PanelApplet *applet)
           (stat("/proc/acpi", &statbuf) == 0)) {
       battstat_error_dialog (
 	       applet,
-	       "Can't access ACPI events in /var/run/acpid.socket!\n\n"
-		 "Make sure the ACPI subsystem is working and\n"
-		 "the acpid daemon is running.");
+	       _("Can't access ACPI events in /var/run/acpid.socket! "
+		 "Make sure the ACPI subsystem is working and "
+		 "the acpid daemon is running."));
       using_acpi = TRUE;
       acpi_count = 0;
   }
@@ -1574,8 +1626,9 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->orienttype = panel_applet_get_orient (applet);
   battstat->panelsize = panel_applet_get_size (applet);
   battstat->horizont = TRUE;
-  
-  glade_gnome_init ();
+  battstat->about_dialog = NULL;
+
+  glade_init ();
   
   load_preferences(battstat);
   create_layout(battstat);
@@ -1602,6 +1655,23 @@ battstat_applet_fill (PanelApplet *applet)
                                      battstat_menu_verbs,
                                      battstat);
 
+  if (panel_applet_get_locked_down (PANEL_APPLET (battstat->applet))) {
+	  BonoboUIComponent *popup_component;
+
+	  popup_component = panel_applet_get_popup_component (PANEL_APPLET (battstat->applet));
+
+	  bonobo_ui_component_set_prop (popup_component,
+					"/commands/BattstatProperties",
+					"hidden", "1",
+					NULL);
+  }
+
+  atk_widget = gtk_widget_get_accessible (battstat->applet);
+  if (GTK_IS_ACCESSIBLE (atk_widget)) {
+	  atk_object_set_name (atk_widget, _("Battery Charge Monitor"));
+	  atk_object_set_description(atk_widget, _("Monitor a laptop's remaining power"));
+  }
+
   return TRUE;
 }
 
