@@ -1,4 +1,4 @@
-/* $Id: weather.c,v 1.71 2004/02/05 23:59:23 dsmit Exp $ */
+/* $Id: weather.c,v 1.74 2004/03/10 22:29:30 dsmit Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -1538,17 +1538,13 @@ static void iwin_start_open (WeatherInfo *info)
     ** file (the PA stands for the state pennsylvania). The url used wants the state
     ** as pa, and the zone as lower case paz021.
     */
-    zone = g_strdup (loc->zone);
-    g_ascii_strdown (zone, strlen (zone));
-    state = g_strdup (zone);
-    state[2] = '\0';
+    zone = g_ascii_strdown (loc->zone, -1);
+    state = g_strndup (zone, 2);
     
     url = g_strdup_printf ("http://weather.noaa.gov/pub/data/forecasts/zone/%s/%s.txt",
         		   state, zone); 
-    if (zone)
-    	g_free (zone);   
-    if (state)
-    	g_free (state);
+    g_free (zone);   
+    g_free (state);
 
     gnome_vfs_async_open(&info->iwin_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, iwin_finish_open, info);
@@ -1999,7 +1995,7 @@ const gchar *weather_info_get_humidity (WeatherInfo *info)
 
 const gchar *weather_info_get_wind (WeatherInfo *info)
 {
-    static gchar buf[100];
+    static gchar buf[200];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
@@ -2017,7 +2013,7 @@ const gchar *weather_info_get_wind (WeatherInfo *info)
 
 const gchar *weather_info_get_pressure (WeatherInfo *info)
 {
-    static gchar buf[30];
+    static gchar buf[100];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
@@ -2029,7 +2025,7 @@ const gchar *weather_info_get_pressure (WeatherInfo *info)
 
 const gchar *weather_info_get_visibility (WeatherInfo *info)
 {
-    static gchar buf[40];
+    static gchar buf[100];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
